#pragma comment(lib, "physfs_static")
#include "physfs.h"
#include <string.h>
#include <stdio.h>

#include "../../emu/h/types.h"
#include "../../emu/h/emuFileHandling.h"
#include "../../emu/h/emuGeneral.h"
#include "../../nes/h/nesGeneral.h"

static char **rc = 0;

char szBack [] = "../";

char **FS_string;
char *FSszFilter = 0;
const PHYSFS_ArchiveInfo **archiveFiles;

static int numOfFolders, numOfFiles;

void FSback();
static void setPath(char *sz);

int getFSListAmount()
{
	return numOfFolders + numOfFiles;
}

static void freeList()
{
	if (rc)
		PHYSFS_freeList(rc);

	rc = 0;
}

int FSenter(int indx, char *sz)
{
	if (indx == 0)
	{
		FSback();
	}
	else if (indx < numOfFolders)
	{
		char sz[2][300];
		strcpy(sz[1], FS_string[indx]);
		freeList();
		rc = PHYSFS_getSearchPath();
		sprintf(sz[0], "%s/%s", rc[0], sz[1]);
		setPath(sz[0]);
	}
	else if ( indx< numOfFolders + numOfFiles)
	{
		sprintf(sz, "%s/%s", PHYSFS_getRealDir(FS_string[indx]), FS_string[indx]);
		return 1;
	}
	return 0;		//Still in a folder
}

static int filterOut(char *sz)
{
	int i, arch, strLen, usingFilters;

	if (FSszFilter == 0)
		usingFilters = 0;
	else
		usingFilters = !( (FSszFilter[0] == 0) && (FSszFilter[1] == 0) );

	strLen = strlen(sz);
	if (usingFilters && strLen <= 4)
		return 0;

	for (i = strLen - 3; i<strLen; i++)
	{
		sz[i] = sz[i] & (-1 ^ 0x20);		//upppercase it so it matches the archive extension case
	}
	i = arch = 0;
	while (true)
	{
		if (strLen > 4)
		{
			while ( archiveFiles[arch] != 0)
			{
				if ( !(strcmp(&sz[strLen - 3],  archiveFiles[arch]->extension)) )
					return 2;
				arch++;
			}
		}
		if (usingFilters)
		{
			i = 0;
			while (true)
			{
				if ( (FSszFilter[i] == 0) && (FSszFilter[i+1] == 0))
					return 0;

				if ( !(strcmp(&sz[strLen - 4],  &FSszFilter[i])) )
					return 1;
				i += strlen( &FSszFilter[i] ) + 1;
			}
			
		}
		else
			return 1;
	}
}

static void setPath(char *sz)
{
	int i, t, fileIndx, folderIndx;
	freeList();
	rc = PHYSFS_getSearchPath();
	PHYSFS_removeFromSearchPath(rc[0]);
	freeList();

	if ( ! PHYSFS_addToSearchPath( sz, 0))
	{
		emuMsg( (char*) PHYSFS_getLastError() );
	}
	rc = PHYSFS_enumerateFiles( "/" );
	
	i = numOfFiles = 0;
	numOfFolders = 1;
	while (rc[i] != 0)
	{
		if ( PHYSFS_isDirectory (rc[i]) )
			numOfFolders++;
		else
		{
			t = filterOut(rc[i]);
			if (t == 2)
				numOfFolders++;
			else if (t == 1)
				numOfFiles++;
		}
		i++;
	}

	if (FS_string)
		delete FS_string;

	FS_string = new char*[numOfFolders + numOfFiles];
	FS_string[0] = szBack;
	folderIndx = 1;
	fileIndx = numOfFolders;
	i = 0;
	while (rc[i] != 0)
	{
		if ( PHYSFS_isDirectory (rc[i]) )
		{
			FS_string[folderIndx++] = rc[i];
		}
		else
		{
			t = filterOut(rc[i]);
			if (t == 2)
				FS_string[folderIndx++] = rc[i];
			else if (t == 1)
				FS_string[fileIndx++] = rc[i];
		}
		i++;
	}
}

void strBack(char *sz)
{
	int i;
	i = strlen(sz);
	while ( !(( sz[i] == '/') || ( sz[i] == '\\')) )
	{
		if (i == 0)
			return;		//Don't want to go to the negitives
		i--;
	}

	sz[i] = 0;
}

void FSback()
{
	char sz[300];
	freeList();
	rc = PHYSFS_getSearchPath();

	if (rc[0] != 0)
		strcpy(sz, rc[0]);
	else
		sz[0] = 0;

	strBack(sz);
	setPath(sz);
}

int getFS_Amount()
{
	return numOfFiles;
}

char *getFS_String(int indx)
{
	if (indx >= numOfFiles)
		return 0;
	else
		return rc[indx];
}

void setFSFilter(char *sz)
{
	int i;
	i = 0;
capFilterAgain:
	while (sz[i] != 0)
	{
		if (sz[i] != '.')
			sz[i] &= -1 ^ 0x20;
		i++;
	}

	if (sz[++i] != 0)
		goto capFilterAgain;

	FSszFilter = sz;
}

int initFS(char *szHome)
{
	if ( PHYSFS_init(szHome) == 0 )
		return 1;

	archiveFiles = PHYSFS_supportedArchiveTypes();	
	setPath(szHome);
	return 0;
}

void uninitFS()
{
	if (FS_string)
		delete FS_string;
	FS_string = 0;
	freeList();
	PHYSFS_deinit();
}



//////////////////////////////



extern char szFilename[260];

int LoadHeader(u8 *header)
{
	PHYSFS_file * f;
	long size;

	if (szFilename[0] == 0)
		return 0;

	char sz[300]; int i;
	{
		strcpy(sz, szFilename);
		strBack(sz);
		i = strlen(sz)+1;
		setPath(sz);
	}

	f = PHYSFS_openRead(&sz[i]);
	if (f == 0)
	{
		emuMsg("Error loading file");
		return 0;
	}
	
	size = PHYSFS_fileLength(f);
	
	PHYSFS_seek(f, 0);
	PHYSFS_read(f, header, 1, 16);
	PHYSFS_close(f);
	return size;
}

void convertTiles();

int loadImage(int size)
{
	PHYSFS_file *f; int i;
	
	i = SetPrgChrMemory();
	if (i)
		return 0;

	imageSize = size;

	char sz[300];
	{
		strcpy(sz, szFilename);
		strBack(sz);
		i = strlen(sz)+1;
		setPath(sz);
	}

	f = PHYSFS_openRead(&sz[i]);
	if (f == 0)
	{
		emuMsg("Error loading file");
		return 0;
	}

	PHYSFS_seek(f, header.trainer ? 512 + 16: 16);
	PHYSFS_read(f, prgromPtr, 1, header.prg * 0x400 * 16);
	PHYSFS_read(f, chrromPtr, 1, header.chr * 0x400 * 8);
	/*
	if (header.footer)
	{
		int iEnd, iEOF;
		fseek(fileId, 0, SEEK_END);
		iEOF = ftell(fileId);
		iEnd = size + ( header.trainer ? 512+16 : 16 );
		header.footerText = new char [iEOF-iEnd+1];
		fseek(fileId, iEnd, SEEK_SET);
		fread(header.footerText, 1, iEOF-iEnd, fileId);
		header.footerText[iEOF-iEnd] = 0;
	}
	*/
	PHYSFS_close(f);
	convertTiles();
	setRegionMode(szFilename);
	return size;
}

void dumpImage()
{
	u8 iheader[16];
	emuMsg("--- Check dumpImage (phyFS) for problems ---");
return;

	PHYSFS_file *f;
	f = PHYSFS_openWrite(szFilename);
	if (f == 0)
	{
		emuMsg("Could not write to file");
		return;
	}
	memset(iheader, 0, sizeof(iheader));
	iheader[0] = 'N'; iheader[1] = 'E'; iheader[2] = 'S'; iheader[3] = 0x1A;
	iheader[4] = header.prg;
	iheader[5] = header.chr;
	iheader[6] = 
		(header.vmirror	?1 :0) |
		(header.sram	?2 :0) |
		//Dont want trainers
		(header.fourScreenMirroring ?8 :0) |
		(header.mapNumber & 0x0F)<<4;
	
	iheader[7] = (header.mapNumber & 0xF0);

	PHYSFS_seek(f, 0);
	PHYSFS_write(f, iheader, 1, 16);
	
	PHYSFS_write    (f, prgromPtr, 1, (header.prg * 0x400 * 16));
	if (header.chr)
		PHYSFS_write(f, chrromPtr, 1, (header.chr * 0x400 * 8));
	
	PHYSFS_close(f);
	if (header.footerText)
	{
		delete [] header.footerText;
		header.footerText = NULL;
		header.footer = 0;
	}
	header.junk = 0;
}

void LoadWram()
{
	PHYSFS_file *f; unsigned int i;
	char szPath[260];

	GetLocalPath("save\\", "sav", szPath, sizeof(szPath));

	f = PHYSFS_openRead(szPath);
	if (f == 0)
		return;

	PHYSFS_seek(f, 0);
	if ((i = PHYSFS_read(f, wram, 1, sramSize)) != sramSize)
		emuMsg("Error in load sram");

	PHYSFS_close(f);
}


void SaveWram()
{
	PHYSFS_file *f;
	char szPath[260];

	if (!header.sram)
		return;

	GetLocalPath("save\\", "sav", szPath, sizeof(szPath));

	f = PHYSFS_openWrite(szPath);
	if (f == 0)
	{
		emuMsg( "Could not save SRAM" );
		return;
	}
	
	PHYSFS_seek(f, 0);						//Set file to the beginning
	if (PHYSFS_write(f, wram, 1, sramSize) != sramSize)
		emuMsg("Error in save sram");

	PHYSFS_close(f);						//done, close it
}
