#include "../../emu/h/types.h"
#include <string.h>
#include "../../emu/h/emuFileHandling.h"
#include "../../nes/h/nesGeneral.h"
#include "../../emu/h/emuGeneral.h"
#include "../../nes/h/ppu.h"
#include "main.h"

char szLocalPath[260];
char *localPath;

char postLoadInfo;

int createFolder(char *szFoldername);

int initMemory()
{
	ram			= new u8 [ramSize];
	wram		= new u8 [wramSize];
	chrRam		= new u8 [chrramSize];
	nameTable	= new u8 [NtSize];
	return 0;
}

void uninitMemory()
{
	if (ram)
	{
		delete [] ram;
		ram = 0;
	}
	if (wram)
	{
		delete [] wram;
		wram = 0;
	}
	if (chrRam)
	{
		delete [] chrRam;
		chrRam = 0;
	}
	if (nameTable)
	{
		delete [] nameTable;
		nameTable = 0;
	}
	if (prgromPtr)
	{
		delete [] prgromPtr;
		prgromPtr = 0;
	}
	if (chrromPtr)
	{
		delete [] chrromPtr;
		chrromPtr = 0;
	}
}

void SetLocalPath(char *sz)
{
	strcpy(szLocalPath, sz);
}

void GetLocalPath(const char *szFolder, const char *szExt, char *szPath, int strSize)
{
	int i;

	strcpy(szPath, szLocalPath);

	i = strlen(szPath);
	while ((szPath[i] != '/') && (szPath[i] != '\\'))
		i--;

	szPath[i+1] = 0;

	i = strlen(szFilename);
	while ((szFilename[i] != '/') && (szFilename[i] != '\\'))
		i--;

	strcat(szPath, szFolder);

	createFolder(szPath);

	strcat(szPath, &szFilename[i+1]);

	i = strlen(szPath);
	while ((szPath[i] != '.') && (i>0))
		i--;

	strcpy(&szPath[i+1], szExt);
}

int SetPrgChrMemory()
{
	prgromPtr	= new u8 [header.prg * 0x400 * 16];
	chrromPtr	= new u8 [header.chr * 0x400 * 8];
	return 0;
}

void deletePrgChrMemory()
{
	if (prgromPtr)
		delete prgromPtr;
	prgromPtr = 0;
	if (chrromPtr)
		delete chrromPtr;
	chrromPtr = 0;
	return;
}

int postLoad()
{
	if (postLoadInfo == 1)
	{
//		dumpImage();
		loadPath();
		return 1;
	}
	return 0;
}

void setRegionMode(int regionNumber)
{
	nesMode = (nesMode & (-1 ^ regionModeAnd)) | regionNumber | resetLoop;
	brunNes = 0;
}


#define MessageBox(a, b, c, d) 0
#define IDYES 6

int checkHeadInfo(int val)
{
	//postLoadInfo = 0;
	if (val & 0x20)
	{
		header.mapNumber &= 0xf;
		if (MessageBox(mainWnd, "DiskDude! rom. Do you want to clean the header (along with other extra/bad data) ? DiskDude usually have bad headers, may need to change header info", wndName, MB_YESNO) == IDYES)
		{
			postLoadInfo = 1;
			return 0;
		}
	}
	else if (val & 0x40)
	{
		if (MessageBox(mainWnd, "Junk in the header (byte 8-F are not zeros), do you want to clean the header?", wndName, MB_YESNO) == IDYES)
		{
			postLoadInfo = 1;
			return 0;
		}
	}
	switch (val & ~(0x20 | 0x40))
	{
		case 0: break;	//its good
		case 2:
			MessageBox(mainWnd, "Invalid Header", wndName, MB_OK);
			return 2;
			break;
		case 3:
			MessageBox(mainWnd, "No Prg Rom", wndName, MB_OK);
			return 3;
			break;
		case 4:
			if (MessageBox(mainWnd, "PrgRom is not a multiple of 2, do you want to continue", wndName, MB_YESNO) != IDYES)
				return 4;
			break;
		case 5:
			if (MessageBox(mainWnd, "Remove Footer? (and other junk data)", wndName, MB_YESNO) == IDYES)
				return 1;
			break;
		case 6:
			if (MessageBox(mainWnd, "Garbage at end of rom?, Do you want to remove it?", wndName, MB_YESNO) == IDYES)
				return 1;
			break;
		case 7:
			MessageBox(mainWnd, "Missing 512 bytes which is the size of a trainer. Try clearing the trainer bit and reloading the image", wndName, MB_OK);
				return 7;
			break;
		case 8:
		case 9:
			MessageBox(mainWnd, "Missing Data", wndName, MB_OK);
			return 9;
			break;
		default:
			MessageBox(mainWnd,"Unknown Return value from checkHeader()", wndName, MB_OK);
			return -1;
			break;
	}
	return 0;
}
/*
	2 Bad header sig
	3 no prg rom ?
	4 prg isnt multiple of 2
	5 average footer found
	6 garbage at end
	7 no trainer but it thinks there is
	8 missing stuff
	9 missing stuff
	D5 DiskDude! <--crap
	D6 junk in header
*/

#ifdef WIN32

	#undef MessageBox
	#include <windows.h>

	int createFolder(char *szFoldername)
	{
		return !CreateDirectory(szFoldername, 0);
	}

#else

	#include "../pc/physfs.h"
	#include "../pc/pcAPI.h"

	int createFolder(char *szFoldername)
	{
		int i;
		char sz[300];
		const char *sz2;

		strcpy(sz, szFoldername);
		i = strlen(sz);
		sz[i-1] = 0;					//makes sure it doesnt end with '/' or '\'
		strBack(sz);

		PHYSFS_setWriteDir(sz);
		i = strlen(sz) + 1;

		if ( ! PHYSFS_mkdir( &sz[i] ) )
			emuMsg("Error creating folder");
		
		return 0;
	}
#endif
